*:**************************************************
*:
*: Procedure file: SCRN_MKR.PRG
*:
*:         System: Generic Screen Prototyping Tool
*:         Author: Alan Schwartz
*:       Copyright (c) 1993, Micromega Systems, Inc.
*:
*:  Procs & Fncts: SCRN_DSGN
*:               : SCRN_EDIT
*:
*:**************************************************
*** SCRN_MKR.prg       Screen Prototyping Utility
****************************************************
* Generic barebones screen manager to support
* prototyping in FoxPro. This program provides the
* ability to to rebuild FoxPro screen programs
* on-the-fly and make changes to screen snippets in
* a UDF procedure file without exiting the
* application.
*
* Usage:
* Replace a call to a screen program (ANYSCRN.SPR)
* and optional UDFS with this prototyping manager
* for instant update access.
* DO ANYSCRN.SPR becomes
* DO SCRN_MKR with "ANYPROJ.PJX", "ANYSCRN.SPR",; 
*    "ANYSCRN.SCX", "ANYUDFS.PRG"
* The screen and procedure files must be EXCLUDED
* from the project.
*
* Parameters:
*
* REQUIRED
* <proj_name>  Char   Name of the project file
*                     (include .PJX)
*
* <scrn_prog>  Char   Name of a screen program
*                     (include .SPR or .PRG)
*
* <scx_name>   Char   Screen object name
*
* OPTIONAL
*
* <udfs_name>  Char   Name of procedure file with
*                     UDF's for the screen
*
* Note: If you use the <udfs_name> option, your
* UDF's file should be constructed like this:
*
* ** Program UDFProc
*    Parameters scrn_prog
*    DO (scrn_prog)  && Pass-thru call to SPR
*      Change this line to DO &SCRN_PROG if you
*      pass parameters to SPR's.
*    RETURN
*    ******
*    FUNCTION UDF1
*    ** code for UDF1 here
*    FUNCTION UDF2
*    ** code for UDF2 here, etc.
*
* (This places the UDF's file in the program call
* stack BEFORE the screen program, so UDF calls from 
* the screen can find the relevant procedures)
****************************************************

PARAMETERS proj_name, scrn_prog, scx_name, udfs_name

ON KEY LABEL alt-f6 DO scrn_dsgn WITH "SCREEN"
IF NOT EMPTY(udfs_name)
   ON KEY LABEL alt-f7 DO scrn_dsgn WITH "UDFS"
ENDIF
** Hotkey assignments arbitrary -- use any keys

wind_name = ""      && Holds name of window onscreen

STORE .F. TO scrn_chng, scrn_udfmod
DO WHILE .T.
   IF NOT EMPTY(udfs_name)
      DO (udfs_name)  WITH scrn_prog
      ** Execute the pass-thru call in the screen's
      ** UDF file, if used
   ELSE
      DO (scrn_prog)
   ENDIF
   IF scrn_chng
      DO scrn_edit                      && See Below
      STORE .F. TO scrn_chng
      LOOP
   ENDIF
   IF scrn_udfmod
      MODI COMM (udfs_name)      && Edit UDF program
      COMPILE (udfs_name)        && Force recompile
      STORE .F. TO scrn_udfmod
      LOOP
   ENDIF
   EXIT
ENDDO
RETURN    && end of main procedure

*!**************************************************
*!
*!      Procedure: SCRN_DSGN
*!
*!      Called by: SCRN_MKR.PRG
*!
*!**************************************************
PROCEDURE SCRN_DSGN
PARAMETERS  mod_what
STORE WOUTPUT() TO wind_name
DO CASE
CASE mod_what = "SCREEN"
   STORE .T. TO scrn_chng
CASE mod_what = "UDFS"
   STORE .T. TO scrn_udfmod
ENDCASE
CLEAR READ
RETURN

*!**************************************************
*!
*!      Procedure: SCRN_EDIT
*!
*!      Called by: SCRN_MKR.PRG
*!
*!
*!**************************************************
PROCEDURE scrn_edit
** Bare-bones prototyping technique

STORE ALIAS() TO was_alias
dummy = ADIR(time_start, scx_name)
MODIFY SCREEN (scx_name)
dummy = ADIR(time_end, scx_name)
IF time_start(3) <> time_end(3) OR ;
   time_start(4) <> time_end(4)
   ** Compare timestamps to see if the screen file
   ** was actually changed
   RELEASE WINDOW (wind_name)
   SELECT 0

   USE (proj_name)
   LOCATE FOR UPPER(scrn_prog) $ UPPER(outfile);
              AND TYPE = "S"
   IF FOUND()
      STORE RECNO() TO scrn_to_gen
      ** Tip: if you save this record pointer and
      ** perform a GOTO instead of the LOCATE
      ** command, subsequent screen changes will be
      ** faster.
   ELSE
      WAIT WINDOW "Screen not found in project"
      RETURN
   ENDIF
   USE
   SELECT (was_alias)
   ** Regenerate screen program
   DO (_GENSCRN) WITH (proj_name), scrn_to_gen
   COMPILE (scrn_prog)
ENDIF
RETURN

*: EOF: SCRN_MKR.PRG
